import { DatabasePaper } from "./database/Database.js";
import { Commands } from "./handlers/command.js";
import { scoreboard } from "./handlers/scoreboard.js";
import { factions } from "./handlers/faction.js";
import { Ban } from "./handlers/Ban.js";
import player from "./handlers/player.js";
import { system, world } from "@minecraft/server";
import { events } from "./events/events.js";
import config from "../config/main.js";
import { item } from "./handlers/item.js";
import { Logs } from "./logger.js";
import { Extra } from "./extra.js";
import { MessageForm } from "./handlers/forms/MessageFormData.js";
import { ActionForm } from "./handlers/forms/ActionFormData.js";
import { ModalForm } from "./handlers/forms/ModalFormData.js";
import { Enchantment } from "./handlers/enchantment.js";
import { Warp } from "./handlers/warp.js";

export const database = new DatabasePaper()
export class ServerClient {
    constructor() {
        this.database = database
        this.command = Commands
        this.server = this.database.register('server')
        this.kit = this.database.register('kit');
        this.settings = this.database.register('settings')
        this.scoreboard = new scoreboard()
        this.ban = new Ban()
        this.factions = new factions()
        this.player = player
        this.extra = new Extra()
        this.item = new item()
        this.log = Logs
        this.messageForm = MessageForm
        this.actionForm = ActionForm
        this.modalForm = ModalForm
        this.enchantment = new Enchantment()
        this.warp = new Warp()
        this.auctions = this.database.register('auctions')
        this.auctionClaims = this.database.register('auctionClaims')
        this.homes = this.database.register('homes')
        this.areas = this.database.register('areas')
    }
    send(message) {
        return world.sendMessage(message)
    }
    on(event, callback) {
        return new events[event]().on(callback);
    }
    off(event) {
        return new events[event]().off();
    }
    reWrite(key, value, category) {
        client.settings.write(key, value)
        if (!category) {
            const reWrite = system.runInterval(() => {
                config[key] = value
                system.clearRun(reWrite)
            }, 1)
        } else {
            const reWrite = system.runInterval(() => {
                config[category][key] = value
                system.clearRun(reWrite)
            }, 1)
        }
    }
}
const client = new ServerClient()
export default client

system.beforeEvents.watchdogTerminate.subscribe(data => {
    data.cancel = true
})