import { database } from "../ServerClient"
import { world } from "@minecraft/server";
import config from "../../config/main";

const factionPrefix = 'faction-'
export class factions {
    constructor() {
        this.database = database.register('factions');
        this.claims = database.register('claims')
    }
    isInFaction(player) {
        return Boolean(player.getTags().find(tag => tag.startsWith(factionPrefix)))
    }
    
    isFactionOwner(player) {
        return Boolean(player.getTags().find(tag => tag.startsWith('factionOwner:')));
    }
    
    getAllMembersOnlineInAFaction(faction) {
        return [...world.getPlayers({tags:[`${factionPrefix}§a${faction}`]})].length
    }
    
    getPlayersFaction(player) {
        const factionTag = player.getTags().find(tag => tag.startsWith(factionPrefix))
        const factionName = factionTag?.replace(factionPrefix, '') 
        return factionName
    }
    
    getPlayersOwnerFaction(player) {
        const factionTag = player.getTags().find(tag => tag.startsWith('factionOwner:'))
        const factionName = factionTag?.replace('factionOwner:', '') 
        return factionName
    }
    
    getPlayersFactionWithNoColors(player) {
        const factionTag = player.getTags().find(tag => tag.startsWith(`${factionPrefix}§a`))
        const factionName = factionTag?.replace(`${factionPrefix}§a`, '') 
        return factionName
    }
    
    newFaction(name, owner) {
        const Data = {
            name: name,
            createdAt: new Date().toLocaleString(),
            owner: owner,
        }
        this.database.write(name, Data)
    }
    getChunk(player) {
        return [~~((player.location.x + 1) / 16), ~~((player.location.z + 1) / 16)]
    }
    getClaimOwner(chunk) { 
        if (!this.claims.has(chunk)) return
        return this.claims.read(chunk).owner
    }
    isChunkClaimed(chunk) {
        return Boolean(this.claims.has(chunk))
    }
    newClaim(owner, claimArea) {
        this.claims.write(claimArea, { owner: owner.name, faction: this.getPlayersFaction(owner) })
        owner.score.removeScore(config.currency, config.claimCost)
    }
}