import { ActionFormData } from "@minecraft/server-ui"
import { system } from "@minecraft/server"

export class ActionForm {
    constructor() {
        this.form = new ActionFormData()
    }
    setTitle(title) {
        system.run(() => this.form.title(title))
        return this
    }
    setBody(body) {
        system.run(() => this.form.body(body))
        return this
    }
    addButton(name, icon) {
        system.run(() => this.form.button(name, icon))
        return this
    }
    show(player, callback) {
        system.run(() => {
            this.form.show(player).then((res) => {
                if (!callback) return
                callback(res, player)
            })
        })
    }
    async force(player, callback, timeout = Infinity) {
        const startTick = system.currentTick;
        while ((system.currentTick - startTick) < timeout) {
            const response = await (this.form.show(player));
            if (response.cancelationReason !== "UserBusy") {
                callback(response)
                return response;
            }
        };
        player.response.error('ActionForm cancelled. Reason: timedOut')
    };
}