import { ModalFormData } from '@minecraft/server-ui'
import { system } from '@minecraft/server'

export class ModalForm {
    constructor() {
        this.form = new ModalFormData()
    }
    setTitle(title) {
        system.run(() => this.form.title(title))
        return this
    }
    addTextField(name, value, defaultValue) {
        system.run(() => this.form.textField(name, value, defaultValue))
        return this
    }
    addDropDown(name, values) {
        system.run(() => this.form.dropdown(name, values))
        return this
    }
    addSlider(name, minValue, maxValue, valueStep, defaultValue) {
        system.run(() => this.form.slider(name, minValue, maxValue, valueStep, defaultValue))
        return this
    }
    addToggle(name, defaultValue) {
        system.run(() => this.form.toggle(name, defaultValue))
        return this
    }
    show(player, callback) {
        system.run(() => {
            this.form.show(player).then((res) => {
                if (!callback) return
                callback(res)
            })
        })
    }
    async force(player, callback, timeout = Infinity) {
        const startTick = system.currentTick;
        while((system.currentTick - startTick) < timeout) {
            const response = await (this.form.show(player));
            if (response.cancelationReason !== "UserBusy") {
                callback(response)
                return response;
            }
        }
        player.response.error('ModalForm cancelled. Reason: timedOut')
    }
}