import { ItemStack, EnchantmentTypes, ItemTypes, Enchantment } from '@minecraft/server'

export class item {
    setLore(player, lore = []) {
        const container = player.getComponent("inventory").container,
        item = container.getItem(player.selectedSlot);
        if (!item) return player.response.error('You must hold an item')
        item.setLore(lore);
        container.setItem(player.selectedSlot, item);
    }
    setName(player, nameTag = []) {
        const container = player.getComponent("inventory").container,
        item = container.getItem(player.selectedSlot);
        if (!item) return player.response.error('You must hold an item')
        item.nameTag = nameTag
        container.setItem(player.selectedSlot, item);
    }
    repair(item) {
        const newItem = new ItemStack(item.typeId, item.amount);
        newItem.nameTag = item.nameTag;
        newItem.getComponents = item.getComponents;
        newItem.setLore(item.getLore());
        newItem.getComponent('enchantments').enchantments = item.getComponent('enchantments').enchantments;
        return newItem;
    }
    getItemName(item) {
        return item.nameTag ?? item.typeId.split(":")[1].split('_').map(v => v[0].toUpperCase() + v.slice(1).toLowerCase()).join(" ")
    }
    getItemData(item) {
        var _a;
        if (!item) return undefined
        const itemData = {
            id: item.typeId,
            amount: item.amount,
            nameTag: item.nameTag,
            lore: item.getLore(),
            enchantments: [],
        };
        if (!item.hasComponent("enchantments"))
            return itemData;
        const enchants = (_a = item.getComponent('enchantments')) === null || _a === void 0 ? void 0 : _a.enchantments;
        if (enchants) {
            [...enchants].forEach(e => {
                itemData.enchantments.push({
                    id: e.type.id,
                    level: e.level,
                });
            })
        }
        return itemData;
    }

    newItem(itemData) {
        const item = new ItemStack(ItemTypes.get(itemData.id), itemData.amount);
        item.nameTag = itemData.nameTag;
        item.setLore(itemData.lore);
        const enchComp = item.getComponent("enchantments");
        const enchants = enchComp === null || enchComp === void 0 ? void 0 : enchComp.enchantments;
        if (enchants) {
            for (let enchant of itemData.enchantments) {
                const key = enchant.id.replace("minecraft:", "")
                const type = EnchantmentTypes.get(key);
                if (!type)
                    continue;   
                enchants.addEnchantment(new Enchantment(type, enchant.level));
            }
            enchComp.enchantments = enchants;
        }
        return item;
    }
}