import client from "../ServerClient"
import { EquipmentSlot } from "@minecraft/server"
import { database } from "../ServerClient"

export class Warp {
    constructor() {
        this.warps = database.register('warps')
        this.warpInventorys = database.register('warpInvntorys')
        this.spawnInventorys =  database.register('spawnInventorys')
    }
    load(player, warp)  {
        player.runCommandAsync('clear @s').then(() => {
            if (!this.warpInventorys.has(player.name + ' ' + warp)) return
            const Data = this.warpInventorys.read(player.name + ' ' + warp), inventory = player.getComponent('inventory').container, equipment = player.getComponent('equipment_inventory')
            if (Data.offhand) equipment.setEquipment(EquipmentSlot.offhand, client.item.newItem(Data.offhand));
            if (Data.armor.helmet) equipment.setEquipment(EquipmentSlot.head, client.item.newItem(Data.armor.helmet));
            if (Data.armor.chest) equipment.setEquipment(EquipmentSlot.chest, client.item.newItem(Data.armor.chest));
            if (Data.armor.legs) equipment.setEquipment(EquipmentSlot.legs, client.item.newItem(Data.armor.legs));
            if (Data.armor.feet) equipment.setEquipment(EquipmentSlot.feet, client.item.newItem(Data.armor.feet));
            if (!Data.items) return
            for (const item of Data?.items) {
                inventory.addItem(client.item.newItem(item));
            }
        })
    }
    save(player, warp) {
        var _a, _b, _c, _d;
        const items = [], inventory = player.getComponent('inventory').container, equipment = player.getComponent('equipment_inventory')
        for (let i = 0; i < inventory.size; i++) {
            const item = inventory.getItem(i);
            if (!item) continue; 
            items.push(client.item.getItemData(item));
        }
        if (!items.length) return this.warpInventorys.delete(player.name + ' ' + warp)
        const offhand = client.item.getItemData(equipment.getEquipment(EquipmentSlot.offhand)), 
        armor = {
            helmet: (_a = client.item.getItemData(equipment.getEquipment(EquipmentSlot.head))) !== null && _a !== void 0 ? _a : undefined,
            chest: (_b = client.item.getItemData(equipment.getEquipment(EquipmentSlot.chest))) !== null && _b !== void 0 ? _b : undefined,
            legs: (_c = client.item.getItemData(equipment.getEquipment(EquipmentSlot.legs))) !== null && _c !== void 0 ? _c : undefined,
            feet: (_d = client.item.getItemData(equipment.getEquipment(EquipmentSlot.feet))) !== null && _d !== void 0 ? _d : undefined
        }
        this.warpInventorys.write(player.name + ' ' + warp, {
            items: items,
            offhand: offhand !== null && offhand !== void 0 ? offhand : undefined,
            armor
        })
    }
    spawnSave(player) {
        var _a, _b, _c, _d;
        const items = [], inventory = player.getComponent('inventory').container, equipment = player.getComponent('equipment_inventory')
        for (let i = 0; i < inventory.size; i++) {
            const item = inventory.getItem(i);
            if (!item) continue; 
            items.push(client.item.getItemData(item));
        }
        if (!items.length) return this.spawnInventorys.delete(player.name)
        const offhand = client.item.getItemData(equipment.getEquipment(EquipmentSlot.offhand)), 
        armor = {
            helmet: (_a = client.item.getItemData(equipment.getEquipment(EquipmentSlot.head))) !== null && _a !== void 0 ? _a : undefined,
            chest: (_b = client.item.getItemData(equipment.getEquipment(EquipmentSlot.chest))) !== null && _b !== void 0 ? _b : undefined,
            legs: (_c = client.item.getItemData(equipment.getEquipment(EquipmentSlot.legs))) !== null && _c !== void 0 ? _c : undefined,
            feet: (_d = client.item.getItemData(equipment.getEquipment(EquipmentSlot.feet))) !== null && _d !== void 0 ? _d : undefined
        }
        this.spawnInventorys.write(player.name, {
            items: items,
            offhand: offhand !== null && offhand !== void 0 ? offhand : undefined,
            armor
        })
    }
    spawnLoad(player) {
        player.runCommandAsync('clear @s').then(() => {
            if (!this.spawnInventorys.has(player.name)) return
            const Data = this.spawnInventorys.read(player.name), inventory = player.getComponent('inventory').container, equipment = player.getComponent('equipment_inventory')
            if (Data.offhand) equipment.setEquipment(EquipmentSlot.offhand, client.item.newItem(Data.offhand));
            if (Data.armor.helmet) equipment.setEquipment(EquipmentSlot.head, client.item.newItem(Data.armor.helmet));
            if (Data.armor.chest) equipment.setEquipment(EquipmentSlot.chest, client.item.newItem(Data.armor.chest));
            if (Data.armor.legs) equipment.setEquipment(EquipmentSlot.legs, client.item.newItem(Data.armor.legs));
            if (Data.armor.feet) equipment.setEquipment(EquipmentSlot.feet, client.item.newItem(Data.armor.feet));
            if (!Data.items) return
            for (const item of Data?.items) {
                inventory.addItem(client.item.newItem(item));
            }
        })
    }
}