import { system, world, EquipmentSlot } from "@minecraft/server";
import client from "./ServerClient";
import { Area } from "./handlers/protect";
import config from "../config/main";

system.runInterval(() => {
    world.getAllPlayers().forEach((player) => {
        //Faction Checker
        if (client.factions.isInFaction(player) && !client.factions.database.has(client.factions.getPlayersFactionWithNoColors(player))) {
            player.removeTag(`faction-${client.factions.getPlayersFaction(player)}`)
        }
        //Combat Checker
        if (player.hasTag('inCombat') && client.scoreboard.get(player, 'inCombat') == 0) {
            player.removeTag('inCombat')
            player.onScreenDisplay.setActionBar('§aYou are no longer in combat')
        }
        //Area Checker
        const area = Area.as(player, [player.location.x, player.location.z])
        if (area.isInArea) {
            if (player.hasTag(config.adminTag)) return;
            player.addTag('protect')
        } else player.removeTag('protect')
        //Faction Entry/Leave Message
        const chunk = client.factions.getChunk(player)
        if (!player.hasTag('inClaim') && client.factions.isChunkClaimed(`${chunk[0]}_${chunk[1]}`)) {
            client.player.send(player, `You have entered ${client.factions.getClaimOwner(`${chunk[0]}_${chunk[1]}`)}'s §r§cfaction`, false, false)
            player.addTag('inClaim')
        } 
        if (player.hasTag('inClaim') && !client.factions.isChunkClaimed(`${chunk[0]}_${chunk[1]}`)) {
            client.player.send(player, `You have entered the wild`, false, false)
            player.removeTag('inClaim')
        }
    })  
})

//Time Checkers, Auction Claims
system.runInterval(() => {
    world.getAllPlayers().forEach(plr => {
        client.auctionClaims.forEach((key, value) => {
            if (key == plr.id) {
                client.scoreboard.add(plr, config.currency, value)
                client.auctionClaims.delete(key)
            } 
        })
        if (client.scoreboard.get(plr, 'pearlTimer') >= 1) {
            client.scoreboard.remove(plr, 'pearlTimer', 1)
        }
        if (client.scoreboard.get(plr, 'inCombat') >= 1) {
            client.scoreboard.remove(plr, 'inCombat', 1)
        }
        if (client.scoreboard.get(plr, 'sents') >= 1) {
            client.scoreboard.remove(plr, 'sents', 1)
        }
        if (client.ban.database.has(plr.name)) {
            const Data = client.ban.database.read(plr.name) 
            plr.runCommandAsync(`kick @s §cYou have been banned from the server By: ${Data.by} Reason: ${Data.reason} At: ${Data.date}`)
        }
    })
}, 20)

//Enchantment System
system.runInterval(() => {
    const tools = ['sword', 'pickaxe', 'shovel', 'hoe', 'axe']
    client.enchantment.database.forEach((_, info) => {
        if (info.event == 'System') {
            world.getPlayers().forEach(plr => {
                const equipment = plr.getComponent('equipment_inventory')
                if (tools.includes(info.equpiedOn)) {
                    const inv = plr.getComponent('inventory').container
                    for (let i = 1; i < info.maxLevel + 1; i++) {
                        if (!inv.getItem(plr.selectedSlot)) return
                        const lore = `${inv.getItem(plr.selectedSlot).getLore()}`.trim().split(/\s+/g);
                        if (lore.includes(client.enchantment.getTierColor(info.tier) + info.name) && lore[lore.indexOf(client.enchantment.getTierColor(info.tier) + info.name) + 1] == client.enchantment.convertToRoman(i) && inv.getItem(plr.selectedSlot).typeId.endsWith(info.equpiedOn)) {
                            try {
                                plr.runCommandAsync(info.command.replace('$level', i - 1))
                            } catch {}
                        }
                    }
                } else {
                    if (!equipment.getEquipment(EquipmentSlot[info.equpiedOn])) return
                    for (let i = 1; i < info.maxLevel + 1; i++) {
                        const lore = `${equipment.getEquipment(EquipmentSlot[info.equpiedOn]).getLore()}`.trim().split(/\s+/g);
                        if (lore.includes(client.enchantment.getTierColor(info.tier) + info.name) && lore[lore.indexOf(client.enchantment.getTierColor(info.tier) + info.name) + 1] == client.enchantment.convertToRoman(i)) { 
                            try {
                                plr.runCommandAsync(info.command.replace('$level', i - 1))
                            } catch {}
                        }
                    }
                }
            })
        }
    })
})