import client from "../../../API/ServerClient"
import { module } from "../../../API/module"
import { EquipmentSlot } from "@minecraft/server"
import config from "../../../config/main"

module.exports({
    module: new client.command()
    .setName('kit')
    .setDescription('A simple kit system')
    .setCategory('miscellaneous')
    .firstArguments(['create', 'remove', 'set', 'buy', 'view', 'reset', 'list'], true)
    .addDynamicArgument('create', 'create', 'name')
    .addDynamicArgument('remove', 'remove', 'name')
    .addDynamicArgument('set', 'set', 'nameset')
    .addDynamicArgument('buy', 'buy', 'name')
    .addDynamicArgument('view', 'view', 'name')
    .addDynamicArgument('reset', 'reset', 'name')
    .addDynamicArgument('list', 'list')
    .addAnyArgument('name', 1)
    .addAnyArgument('nameset', 1, {}, 'permission')
    .addAnyArgument('permission', 1, {}, 'price')
    .addNumberArgument('price'),
    execute(ctx) {
        ctx.execute((sender) => sender.management.jailed && sender.response.error('You cant use this command while your jailed'))
        ctx.executeArgument('create', (sender, _, args) => {
            if (!sender.permission.hasPermission('admin')) return sender.response.error(`You need admin permissions to use this argument`)
            if (client.kit.has(args[0])) return sender.response.error('That kit already exists')
            client.kit.write(args[0], args[0])
            sender.response.send(`Created the kit with the name §6${args[0]}`)
        })
        ctx.executeArgument('remove', (sender, _, args) => {
            if (!sender.permission.hasPermission('admin')) return sender.response.error(`You need admin permissions to use this argument`)
            if (!client.kit.has(args[0])) return sender.response.error('That kit dosent exist')
            client.kit.delete(args[0]);
            sender.response.send(`You have removed the kit §6${args[0]} §csuccessfuly`)
        })
        ctx.executeArgument('set', (sender, _, args) => {
            if (!sender.permission.hasPermission('admin')) return sender.error(`You need admin permissions to use this argument`)
            const inventory = sender.getComponent('inventory').container;
            let permission, price;
            var _a, _b, _c, _d;
            if (!client.kit.has(args[0])) return sender.response.error('That kit dosent exist')
            if (!args[1]) return sender.response.error('You must provide a permission for the kit, type §6none §cafter this argument if you dont want a permission for the kit')
            permission = args[1]
            if (!args[2] && args[2] !== 0) return sender.response.error('You must provide a price for the kit')
            price = args[2]
            const items = [];
            let itemCount = 0;
            for (let i = 0; i < inventory.size; i++) {
                const item = inventory.getItem(i);
                if (!item) continue; 
                itemCount++;
                items.push(client.item.getItemData(item));
            }
            if (!items.length)
                return sender.response.error('There were no items to add to the kit')
            const equipment = sender.getComponent('equipment_inventory'), 
            offhand = client.item.getItemData(equipment.getEquipment(EquipmentSlot.offhand)), 
            armor = {
                helmet: (_a = client.item.getItemData(equipment.getEquipment(EquipmentSlot.head))) !== null && _a !== void 0 ? _a : undefined,
                chest: (_b = client.item.getItemData(equipment.getEquipment(EquipmentSlot.chest))) !== null && _b !== void 0 ? _b : undefined,
                legs: (_c = client.item.getItemData(equipment.getEquipment(EquipmentSlot.legs))) !== null && _c !== void 0 ? _c : undefined,
                feet: (_d = client.item.getItemData(equipment.getEquipment(EquipmentSlot.feet))) !== null && _d !== void 0 ? _d : undefined
            };
            const data = {
                name: args[0],
                itemCount,
                permission: permission ?? "none",
                price: price ?? 0,
                items: items,
                offhand: offhand !== null && offhand !== void 0 ? offhand : undefined,
                armor,
                createdAt: new Date().toLocaleString()
            };
            sender.response.send(`You have set the items in your inventory to the kit §6${args[0]} §cwith the permision: §6${data.permission} §cwith the price: §6${client.extra.parseNumber(data.price)}`)
            client.kit.write(args[0], data);
        })
        ctx.executeArgument('buy', (sender, _, args) => {
            if (!client.kit.has(args[0])) return sender.response.error('That kit dosent exist')
            const inventory = sender.getComponent('inventory').container;
            const equipment = sender.getComponent('equipment_inventory')
            const Data = client.kit.read(args[0])
            if (!args[0].includes(Data.name || Data.createdAt || Data.items)) return sender.response.error('That kit cant be bought')
            if (Data.permission !== "none" && (!player.hasTag(Data.permission))) return sender.response.error('You dont have permission to buy this kit')
            if (sender.score.getScore(config.currency) < Data.price) return sender.response.error(`You need §6${client.extra.parseNumber(Data.price - sender.score.getScore(config.currency))} §cmore to buy this item`)
            const items = Data.items;
            if (inventory.emptySlotsCount < Data.itemCount) return sender.response.error('You dont have enough space to buy this kit')
            if (Data.offhand && equipment.getEquipment(EquipmentSlot.offhand)) return sender.response.error('You must not have any offhand equipments')
            if (Data.armor.helmet && equipment.getEquipment(EquipmentSlot.head)) return sender.response.error('You must not have any helmets equiped')
            if (Data.armor.chest && equipment.getEquipment(EquipmentSlot.chest)) return sender.response.error('You cant must not have any chestplates equiped')
            if (Data.armor.legs && equipment.getEquipment(EquipmentSlot.legs)) return sender.response.error('You must not have any leggings equiped')
            if (Data.armor.feet && equipment.getEquipment(EquipmentSlot.feet)) return sender.response.error('You must not have any boots equiped')
            if (Data.offhand)
                equipment.setEquipment(EquipmentSlot.offhand, client.item.newItem(Data.offhand));
            if (Data.armor.helmet)
                equipment.setEquipment(EquipmentSlot.head, client.item.newItem(Data.armor.helmet));
            if (Data.armor.chest)
                equipment.setEquipment(EquipmentSlot.chest, client.item.newItem(Data.armor.chest));
            if (Data.armor.legs)
                equipment.setEquipment(EquipmentSlot.legs, client.item.newItem(Data.armor.legs));
            if (Data.armor.feet)
                equipment.setEquipment(EquipmentSlot.feet, client.item.newItem(Data.armor.feet));
            for (const item of items) {
                inventory.addItem(client.item.newItem(item));
            }
            sender.response.send(`You have bought the kit §6${args[0]}`)
            sender.score.removeScore(config.currency, Data.price)
        })
        ctx.executeArgument('view', (sender, _, args) => {
            if (!client.kit.has(args[0])) return sender.response.error('That kit dosent exist')
            const Data = client.kit.read(args[0])
            if (!args[0].includes(Data.name || Data.createdAt || Data.items)) return sender.response.error('That kit cant be viewed')
            let tx = "\n"
            tx += `§cName: ${Data.name}\n§cItem Amount: ${Data.itemCount}\n§cPermission: ${Data.permission}\n§cPrice: ${client.extra.parseNumber(Data.price)}\n§cCreated At: ${Data.createdAt}`
            sender.response.send(`§cViewing the kit §6${args[0]}§c:${tx}`, true, false)
        })
        ctx.executeArgument('reset', (sender, _, args) => {
            if (!sender.permission.hasPermission('admin')) return sender.response.error(`You need admin permissions to use this argument`)
            if (!client.kit.has(args[0])) return sender.response.error('That kit dosent exist')
            client.kit.write(args[0], args[0])
            sender.response.send(`You have reseted the kit §6${args[0]} §csuccessfully`)
        })
        ctx.executeArgument('list', (sender) => {
            sender.response.send(`§c----------------\nAvailable kits:${client.kit.allKeys()}\n§c----------------`, true, false)
        })
    }
})