import { system, world } from "@minecraft/server"
import client from "../../API/ServerClient"
import config from "../../config/main"

world.getDimension('overworld').runCommandAsync('gamerule sendcommandfeedback false')
world.getDimension('overworld').runCommandAsync('gamerule commandblockoutput false')
try {
    world.scoreboard.addObjective(config.currency, `§c§l${config.currency}`)
    world.scoreboard.addObjective('pearlTimer', '')
    world.scoreboard.addObjective('inCombat', '')
    world.scoreboard.addObjective('sents', '')
} catch (e) { }

//Item Names Display
system.runInterval(() => {
    if (config.itemNamesDisplay) for (const entity of world.getDimension('overworld').getEntities({ type: 'item' })) entity.nameTag = `§6${entity.getComponent("item").itemStack.amount}x§r ${client.item.getItemName(entity.getComponent("item").itemStack)}`
})

//Name Ranks, Heart Display, Nicknames, Color Names
system.runInterval(() => {
    world.getAllPlayers().forEach((player) => {
        const getRanks = (player) => {
            const ranks = player.getTags().map((v) => {
                if (!v.startsWith('rank:')) return null;
                return v.substring('rank:'.length);
            }).filter((x) => x);
            return ranks.length == 0 ? [config.defaultRank] : ranks.join("§r§8][§r");
        }
        const health = player.getComponent("health")
        const ranks = (plr) => `§r§8[§r${getRanks(plr)}§r§8] `
        player.nameTag = `${config.ranks ? ranks(player) : ''}§r${player.getTags().find(tag => tag.startsWith('nickname:'))?.replace('nickname:', '') ?? client.player.hasColor(player, 'name') ? client.player.colorize(player.getTags().find(tag => tag.startsWith('nickname:'))?.replace('nickname:', '') ?? player.name, client.player.getColor(player, 'name')) : player.name}\n§4❤${Math.round(health.currentValue) / 2}`;
    })
})

//Illeagal Enchantments
system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        if (!player.hasTag(config.adminTag) && config.AntiCheat.illegalEnchantments) {
            const { container } = player.getComponent("inventory");
            if (container.size === container.emptySlotsCount) return;
            for (let i = 0; i < container.size; i++) {
                const item = container.getItem(i);
                if (!item) continue;
                const { enchantments } = item.getComponent("enchantments");
                const enchantmentIterator = enchantments[Symbol.iterator]();
                let set = false;
                for (let object = enchantmentIterator.next(); !object?.done; object = enchantmentIterator.next()) {
                    const { value: enchant } = object;
                    const { type: { maxLevel }, level } = enchant;
                    if (level <= maxLevel) continue;
                    enchantments.removeEnchantment(enchant.type);
                    item.getComponent("enchantments").enchantments = enchantments;
                    set = true;
                }
                if (!set) continue;
                container.setItem(i, item);
                new client.log({
                    from: player.name,
                    translate: 'AntiCheat',
                    reason: 'illegal enchantments',
                    warn: true
                })
            }
        }
    })
})

//Leaderboard System
system.runInterval(() => {
    Array.from(world.getDimension('overworld').getEntities({ type: 'mod:ft', tags: ['ftlb'] })).forEach(entity => {
        const objective = entity.getTags().find(t => t.startsWith('ft:')).replace('ft:', '');
        if (!objective || !world.scoreboard.getObjective(objective)) entity.nameTag = `§c§lObjective: ${objective} has no records`;
        let leaderboard = [];
        let players = []
        world.getAllPlayers().map((plr) => {
            for (let i = 0; i < world.getAllPlayers().length; i++) {
                if (players.includes(plr.name)) continue;
                leaderboard.push(`§b#${players.length + 1}§c ${plr.name}§e ${client.scoreboard.get(plr, objective) == undefined ? 0 : client.extra.parseNumber(client.scoreboard.get(plr, objective))}`)
                players.push(plr.name)
            }
        })
        if (!leaderboard) return;
        leaderboard = leaderboard.slice(0, parseInt(entity.getTags().find(t => t.startsWith('ftl:')).replace('ftl:', '')));
        leaderboard.unshift(entity.getTags().find(t => t.startsWith('fth:')).replace('fth:', ''));
        entity.nameTag = leaderboard.join('\n');
    })
}, 20)